﻿namespace ProjectCleaningCar;
/// <summary>
/// Класс, отвечающий за прорисовку и перемещение объекта сущности
/// </summary>
public class DrawningCleaningCar
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityCleaningCar? EntityCleaningCar { get; private set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки машины
    /// </summary>
    private int? _startPosX;
    /// <summary>
    /// Верхняя кооридната прорисовки машины
    /// </summary>
    private int? _startPosY;
    /// <summary>
    /// Ширина прорисовки машины
    /// </summary>
    private readonly int _drawningCarWidth = 130;
    /// <summary>
    /// Высота прорисовки машины
    /// </summary>
    private readonly int _drawningCarHeight = 90;
    /// <summary>
    /// Инициализация свойств
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="tank">Бак с водой</param>
    /// <param name="sweepingBrush">Подметательная щётка</param>
    /// <param name="flashlight">Проблескового маячок</param>
    public void Init(int speed, double weight, Color bodyColor, Color
additionalColor, bool tank, bool sweepingBrush, bool flashlight)
    {
        EntityCleaningCar = new EntityCleaningCar();
        EntityCleaningCar.Init(speed, weight, bodyColor, additionalColor,
tank, sweepingBrush, flashlight);
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }
    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя 
    /// разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {

        if (width < _drawningCarWidth || height < _drawningCarHeight) { return false; };
        _pictureWidth = width;
        _pictureHeight = height;
        if (_startPosX != null || _startPosY != null)
        {
            if (_startPosX + _drawningCarWidth > _pictureWidth)
            {
                _startPosX = -_drawningCarWidth + _pictureWidth;
            }
            else if (_startPosX < 0)
            {
                _startPosX = 0;
            }
            if (_startPosY + _drawningCarHeight > _pictureHeight)
            {
                _startPosY = -_drawningCarHeight + _pictureHeight;
            }
            else if (_startPosY < 0)
            {
                _startPosY = 0;
            }
        }
        return true;
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Кордината Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }

        if (x + _drawningCarWidth > _pictureWidth)
        {
            _startPosX = _pictureWidth - _drawningCarWidth;
        }
        else if (x < 0)
        {
            _startPosX = 0;
        }
        else
        {
            _startPosX = x;
        }

        if (y + _drawningCarHeight > _pictureHeight)
        {
            _startPosY = _pictureHeight - _drawningCarHeight;
        }
        else if (y < 0)
        {
            _startPosY = 0;
        }
        else
        {
            _startPosY = y;
        }

    }
    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещение выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityCleaningCar == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityCleaningCar.Step > 0)
                {
                    _startPosX -= (int)EntityCleaningCar.Step;
                }
                return true;
            //вправо
            case DirectionType.Right:
                if (_startPosX.Value + _drawningCarWidth + EntityCleaningCar.Step < _pictureWidth)
                {
                    _startPosX += (int)EntityCleaningCar.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityCleaningCar.Step > 0)
                {
                    _startPosY -= (int)EntityCleaningCar.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + _drawningCarHeight + EntityCleaningCar.Step < _pictureHeight)
                {
                    _startPosY += (int)EntityCleaningCar.Step;
                }
                return true;
            default:
                return false;

        }
    }
    /// <summary>
    /// Отрисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public void DrawTransport(Graphics g)
    {
        if (EntityCleaningCar == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);
        Brush bodyBrush = new SolidBrush(EntityCleaningCar.BodyColor);
        Brush additionalBrush = new SolidBrush(EntityCleaningCar.AdditionalColor);
        //бак
        if (EntityCleaningCar.Tank)
        {
            g.FillRectangle(additionalBrush, _startPosX.Value, _startPosY.Value, _drawningCarWidth - 32, 38);
            g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value, _drawningCarWidth - 32, 38);
            g.FillRectangle(bodyBrush, _startPosX.Value, _startPosY.Value + 10, _drawningCarWidth - 32, 10);
            g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 10, _drawningCarWidth - 32, 10);
        }
        //подметательная щётка
        if (EntityCleaningCar.SweepingBrush)
        {
            Pen additionalPen = new(EntityCleaningCar.AdditionalColor);
            g.DrawEllipse(additionalPen, _startPosX.Value + _drawningCarWidth - 5, _startPosY.Value + 50, 18, 18);
            g.FillEllipse(additionalBrush, _startPosX.Value + _drawningCarWidth - 5, _startPosY.Value + 50, 18, 18);
            g.FillEllipse(bodyBrush, _startPosX.Value + _drawningCarWidth - 1, _startPosY.Value + 54, 10, 10);
            g.DrawLine(pen, _startPosX.Value + _drawningCarWidth - 10, _startPosY.Value + 50, _startPosX.Value + _drawningCarWidth + 5, _startPosY.Value + 60);

        }
        //проблесковый маячок
        if (EntityCleaningCar.Flashlight)
        {
            g.FillRectangle(additionalBrush, _startPosX.Value + _drawningCarWidth - 25, _startPosY.Value + 5, 5, 7);
        }
        //основание
        g.FillRectangle(bodyBrush, _startPosX.Value, _startPosY.Value + 40, _drawningCarWidth - 10, 10);
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 40, _drawningCarWidth - 10, 10);
        //
        g.FillRectangle(bodyBrush, _startPosX.Value + _drawningCarWidth - 30, _startPosY.Value + 10, 20, 30);
        g.DrawRectangle(pen, _startPosX.Value + _drawningCarWidth - 30, _startPosY.Value + 10, 20, 30);
        //колёса
        g.FillEllipse(bodyBrush, _startPosX.Value + 10, _startPosY.Value + 50, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 10, _startPosY.Value + 50, 20, 20);
        g.FillEllipse(bodyBrush, _startPosX.Value + 35, _startPosY.Value + 50, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 35, _startPosY.Value + 50, 20, 20);
        g.FillEllipse(bodyBrush, _startPosX.Value + 97, _startPosY.Value + 50, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 97, _startPosY.Value + 50, 20, 20);
        //диски
        g.FillEllipse(additionalBrush, _startPosX.Value + 15, _startPosY.Value + 55, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 15, _startPosY.Value + 55, 10, 10);
        
        g.FillEllipse(additionalBrush, _startPosX.Value + 40, _startPosY.Value + 55, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 40, _startPosY.Value + 55, 10, 10);
        
        g.FillEllipse(additionalBrush, _startPosX.Value + 102, _startPosY.Value + 55, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 102, _startPosY.Value + 55, 10, 10);
        //окно
        g.DrawRectangle(pen, _startPosX.Value + _drawningCarWidth - 30, _startPosY.Value + 10, 20, 30);
        g.FillRectangle(additionalBrush, _startPosX.Value + _drawningCarWidth - 22, _startPosY.Value + 16, 10, 11);

       
    }
}
